<?php

/**
 * @file
 * Contains \Drupal\per_user_block\Plugin\Condition\PerUser.
 */

namespace Drupal\per_user_block\Plugin\Condition;

use Drupal\Core\Condition\ConditionPluginBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a 'Per User' condition.
 *
 * @Condition(
 *   id = "per_user",
 *   label = @Translation("Per User"),
 *   context = {
 *     "user" = @ContextDefinition("entity:user", label = @Translation("User")),
 *     "block" = @ContextDefinition("entity:block", label = @Translation("Block"))
 *   }
 * )
 *
 */
class PerUser extends ConditionPluginBase {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['custom'] = array(
      '#type' => 'radios',
      '#title' => t('Customizable per user'),
      '#description' => t('Allow individual users to customize the visibility of this block in their account settings.'),
      '#default_value' => isset($this->configuration['custom']) ? $this->configuration['custom'] : PER_USER_BLOCK_CUSTOM_FIXED,
      '#options' => array(
        PER_USER_BLOCK_CUSTOM_FIXED => t('Not customizable'),
        PER_USER_BLOCK_CUSTOM_ENABLED => t('Customizable, visible by default'),
        PER_USER_BLOCK_CUSTOM_DISABLED => t('Customizable, hidden by default'),
      ),
    );
    // Add our JS to display the default description on our vertical tab.
    $form['#attached']['js'][] = drupal_get_path('module', 'per_user_block') . '/js/per_user_block.js';
    return parent::buildConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return array(
      'custom' => PER_USER_BLOCK_CUSTOM_FIXED,
    ) + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['custom'] = $form_state->getValue('custom');
    parent::submitConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function summary() {
    return $this->configuration['custom'] == PER_USER_BLOCK_CUSTOM_FIXED ? t('Block is not customizable') : t('Block is customizable');
  }

  /**
   * {@inheritdoc}
   */
  function evaluate() {
    $visibility = $this->configuration['custom'];

    // If the block position is fixed then it's always visible.
    if ($visibility == PER_USER_BLOCK_CUSTOM_FIXED) {
      return TRUE;
    }

    $block = $this->getContextValue('block');
    $user_block_settings = \Drupal::service('user.data')->get('per_user_block', $this->getContextValue('user')->id(), 'blocks');

    // Use the on/off flag stored in the user settings if they have one.
    if (isset($user_block_settings[$block->id()])) {
      return (bool) $user_block_settings[$block->id()];
    }

    // Use the default selected visibility.
    return $visibility == PER_USER_BLOCK_CUSTOM_ENABLED;
  }

}
